@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Free42 -- an HP-42S calculator simulator
- * Copyright (C) 2004-2024  Thomas Okken
+ * Copyright (C) 2004-2025  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -373,105 +373,152 @@
 #define CMD_LSTO        331
 #define CMD_SST_UP      332
 #define CMD_SST_RT      333
-#define CMD_WSIZE       334
-#define CMD_WSIZE_T     335
-#define CMD_YMD         336
-#define CMD_BSIGNED     337
-#define CMD_BWRAP       338
-#define CMD_BRESET      339
-#define CMD_GETKEY1     340
-#define CMD_GETKEYA     341
-#define CMD_LASTO       342
-#define CMD_LCLV        343
+#define CMD_YMD         334
+#define CMD_GETKEY1     335
+#define CMD_GETKEYA     336
+#define CMD_LASTO       337
+#define CMD_LCLV        338
+/* BASE enhancements */
+#define CMD_SL          339
+#define CMD_SR          340
+#define CMD_RL          341
+#define CMD_RR          342
+#define CMD_RLC         343
+#define CMD_RRC         344
+#define CMD_LJ          345
+#define CMD_RJ          346
+#define CMD_ASR         347
+#define CMD_RLN         348
+#define CMD_RRN         349
+#define CMD_RLCN        350
+#define CMD_RRCN        351
+#define CMD_SB          352
+#define CMD_CB          353
+#define CMD_B_T         354
+#define CMD_NUM_B       355
+#define CMD_MASKL       356
+#define CMD_MASKR       357
+#define CMD_SC          358
+#define CMD_CC          359
+#define CMD_C_T         360
+#define CMD_N_TO_BS     361
+#define CMD_N_TO_BD     362
+#define CMD_N_TO_BQ     363
+#define CMD_N_TO_DS     364
+#define CMD_N_TO_DD     365
+#define CMD_N_TO_DQ     366
+#define CMD_BS_TO_N     367
+#define CMD_BD_TO_N     368
+#define CMD_BQ_TO_N     369
+#define CMD_DS_TO_N     370
+#define CMD_DD_TO_N     371
+#define CMD_DQ_TO_N     372
+#define CMD_WSIZE       373
+#define CMD_WSIZE_T     374
+#define CMD_BSIGNED     375
+#define CMD_BWRAP       376
+#define CMD_BRESET      377
+#define CMD_DECINT      378
+#define CMD_BINSEP      379
+#define CMD_OCTSEP      380
+#define CMD_DECSEP      381
+#define CMD_HEXSEP      382
 /* Useful X-Fcn functions missing from the 42S */
-#define CMD_ANUM        344
-#define CMD_X_SWAP_F    345
-#define CMD_RCLFLAG     346
-#define CMD_STOFLAG     347
+#define CMD_ANUM        383
+#define CMD_X_SWAP_F    384
+#define CMD_RCLFLAG     385
+#define CMD_STOFLAG     386
 /* User-defined functions */
-#define CMD_FUNC        348
-#define CMD_ERRMSG      349
-#define CMD_ERRNO       350
-#define CMD_RTNYES      351
-#define CMD_RTNNO       352
-#define CMD_RTNERR      353
-#define CMD_STRACE      354
+#define CMD_FUNC        387
+#define CMD_ERRMSG      388
+#define CMD_ERRNO       389
+#define CMD_RTNYES      390
+#define CMD_RTNNO       391
+#define CMD_RTNERR      392
+#define CMD_STRACE      393
 /* Big Stack */
-#define CMD_4STK        355
-#define CMD_L4STK       356
-#define CMD_NSTK        357
-#define CMD_LNSTK       358
-#define CMD_DEPTH       359
-#define CMD_DROP        360
-#define CMD_DROPN       361
-#define CMD_DUP         362
-#define CMD_DUPN        363
-#define CMD_PICK        364
-#define CMD_UNPICK      365
-#define CMD_RDNN        366
-#define CMD_RUPN        367
+#define CMD_4STK        394
+#define CMD_L4STK       395
+#define CMD_NSTK        396
+#define CMD_LNSTK       397
+#define CMD_DEPTH       398
+#define CMD_DROP        399
+#define CMD_DROPN       400
+#define CMD_DUP         401
+#define CMD_DUPN        402
+#define CMD_PICK        403
+#define CMD_UNPICK      404
+#define CMD_RDNN        405
+#define CMD_RUPN        406
 /* Miscellaneous */
-#define CMD_NOP         368
-#define CMD_FMA         369
-#define CMD_PGMMENU     370
-#define CMD_PMEXEC      371
-#define CMD_PGMVAR      372
-#define CMD_VARMNU1     373
-#define CMD_X2LINE      374
-#define CMD_A2LINE      375
-#define CMD_A2PLINE     376
-#define CMD_RCOMPLX     377
-#define CMD_PCOMPLX     378
-#define CMD_CAPS        379
-#define CMD_MIXED       380
-#define CMD_SKIP        381
-#define CMD_CPXMAT_T    382
-#define CMD_TYPE_T      383
-#define CMD_A_THRU_F_2  384
-#define CMD_DROP_CANCL  385
-#define CMD_PRREG       386
-#define CMD_CSLD_T      387
-#define CMD_C_LN_1_X    388
-#define CMD_C_E_POW_X_1 389
+#define CMD_NOP         407
+#define CMD_FMA         408
+#define CMD_PGMMENU     409
+#define CMD_PMEXEC      410
+#define CMD_PGMVAR      411
+#define CMD_VARMNU1     412
+#define CMD_X2LINE      413
+#define CMD_A2LINE      414
+#define CMD_A2PLINE     415
+#define CMD_RCOMPLX     416
+#define CMD_PCOMPLX     417
+#define CMD_CAPS        418
+#define CMD_MIXED       419
+#define CMD_SKIP        420
+#define CMD_CPXMAT_T    421
+#define CMD_TYPE_T      422
+#define CMD_A_THRU_F_2  423
+#define CMD_DROP_CANCL  424
+#define CMD_PRREG       425
+#define CMD_CSLD_T      426
+#define CMD_C_LN_1_X    427
+#define CMD_C_E_POW_X_1 428
+#define CMD_GETMI       429
+#define CMD_PUTMI       430
+#define CMD_GETLI       431
+#define CMD_PUTLI       432
+#define CMD_LOCK        433
+#define CMD_UNLOCK      434
+#define CMD_IDENT       435
 /* String & List Functions */
-#define CMD_XSTR        390
-#define CMD_XASTO       391
-#define CMD_LXASTO      392
-#define CMD_XVIEW       393
-#define CMD_APPEND      394
-#define CMD_EXTEND      395
-#define CMD_SUBSTR      396
-#define CMD_LENGTH      397
-#define CMD_HEAD        398
-#define CMD_REV         399
-#define CMD_POS         400
-#define CMD_S_TO_N      401
-#define CMD_N_TO_S      402
-#define CMD_NN_TO_S     403
-#define CMD_C_TO_N      404
-#define CMD_N_TO_C      405
-#define CMD_LIST_T      406
-#define CMD_NEWLIST     407
-#define CMD_TO_LIST     408
-#define CMD_FROM_LIST   409
+#define CMD_XSTR        436
+#define CMD_XASTO       437
+#define CMD_LXASTO      438
+#define CMD_XVIEW       439
+#define CMD_APPEND      440
+#define CMD_EXTEND      441
+#define CMD_SUBSTR      442
+#define CMD_LENGTH      443
+#define CMD_HEAD        444
+#define CMD_REV         445
+#define CMD_POS         446
+#define CMD_S_TO_N      447
+#define CMD_N_TO_S      448
+#define CMD_NN_TO_S     449
+#define CMD_C_TO_N      450
+#define CMD_N_TO_C      451
+#define CMD_LIST_T      452
+#define CMD_NEWLIST     453
+#define CMD_TO_LIST     454
+#define CMD_FROM_LIST   455
 /* Generalized Comparisons */
-#define CMD_X_EQ_NN     410
-#define CMD_X_NE_NN     411
-#define CMD_X_LT_NN     412
-#define CMD_X_GT_NN     413
-#define CMD_X_LE_NN     414
-#define CMD_X_GE_NN     415
-#define CMD_0_EQ_NN     416
-#define CMD_0_NE_NN     417
-#define CMD_0_LT_NN     418
-#define CMD_0_GT_NN     419
-#define CMD_0_LE_NN     420
-#define CMD_0_GE_NN     421
+#define CMD_X_EQ_NN     456
+#define CMD_X_NE_NN     457
+#define CMD_X_LT_NN     458
+#define CMD_X_GT_NN     459
+#define CMD_X_LE_NN     460
+#define CMD_X_GE_NN     461
+#define CMD_0_EQ_NN     462
+#define CMD_0_NE_NN     463
+#define CMD_0_LT_NN     464
+#define CMD_0_GT_NN     465
+#define CMD_0_LE_NN     466
+#define CMD_0_GE_NN     467
 /* For Plus42 compatibility */
-#define CMD_WIDTH       422
-#define CMD_HEIGHT      423
+#define CMD_WIDTH       468
+#define CMD_HEIGHT      469
 
-#define CMD_SENTINEL    424
+#define CMD_SENTINEL    470
 
 
 /* command_spec.argtype */
@@ -491,8 +538,10 @@
 #define ARG_PRGM     12 /* Alpha label (CATSECT_PGM) */
 #define ARG_RVAR     13 /* Variable (real only) (MVAR, INTEG, SOLVE) */
 #define ARG_MAT      14 /* Variable (matrix only) (EDITN, INDEX) */
-#define ARG_XSTR     15 /* Long string (XSTR) */
-#define ARG_OTHER    16 /* Weirdos */
+#define ARG_M_STK    15 /* Matrix variable or stack (GETMI, PUTMI) */
+#define ARG_L_STK    16 /* List variable or stack (GETLI, PUTLI) */
+#define ARG_XSTR     17 /* Long string (XSTR) */
+#define ARG_OTHER    18 /* Weirdos */
 
 
 /* command_spec.flags */
